@@ -65,6 +65,7 @@ QBluetoothDeviceDiscoveryAgentPrivate::QBluetoothDeviceDiscoveryAgentPrivate(con
 QBluetoothDeviceDiscoveryAgentPrivate::~QBluetoothDeviceDiscoveryAgentPrivate()
 {
     delete manager;
+    delete adapter;
 }
 
 bool QBluetoothDeviceDiscoveryAgentPrivate::isActive() const
@@ -115,9 +116,23 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start()
     propertiesReply.waitForFinished();
     if(propertiesReply.isError()) {
         errorString = propertiesReply.error().message();
+        delete adapter;
+        adapter = 0;
         qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << "ERROR: " << errorString;
         lastError = QBluetoothDeviceDiscoveryAgent::InputOutputError;
         Q_Q(QBluetoothDeviceDiscoveryAgent);
+        delete adapter;
+        adapter = 0;
+        emit q->error(lastError);
+        return;
+    }
+
+    if (!propertiesReply.value().value(QStringLiteral("Powered")).toBool()) {
+        qCDebug(QT_BT_BLUEZ) << "Aborting device discovery due to offline Bluetooth Adapter";
+        lastError = QBluetoothDeviceDiscoveryAgent::PoweredOffError;
+        errorString = QBluetoothDeviceDiscoveryAgent::tr("Device is powered off");
+        delete adapter;
+        adapter = 0;
         emit q->error(lastError);
         return;
     }
@@ -151,7 +166,7 @@ void QBluetoothDeviceDiscoveryAgentPrivate::_q_deviceFound(const QString &addres
 {
     const QBluetoothAddress btAddress(address);
     const QString btName = dict.value(QLatin1String("Name")).toString();
-    quint32 btClass = dict.value(QLatin1String("Class")).toUInt();    
+    quint32 btClass = dict.value(QLatin1String("Class")).toUInt();
 
     qCDebug(QT_BT_BLUEZ) << "Discovered: " << address << btName
              << "Num UUIDs" << dict.value(QLatin1String("UUIDs")).toStringList().count()
@@ -190,7 +205,7 @@ void QBluetoothDeviceDiscoveryAgentPrivate::_q_deviceFound(const QString &addres
 
 void QBluetoothDeviceDiscoveryAgentPrivate::_q_propertyChanged(const QString &name,
                                                                const QDBusVariant &value)
-{    
+{
     qCDebug(QT_BT_BLUEZ) << Q_FUNC_INFO << name << value.variant();
 
     if (name == QLatin1String("Discovering") && !value.variant().toBool()) {
