@@ -45,13 +45,11 @@
 #include "bluez/manager_p.h"
 #include "bluez/adapter_p.h"
 #include "bluez/device_p.h"
+#include "bluez/bluez_data_p.h"
 
 #include <qplatformdefs.h>
 
 #include <QtCore/QLoggingCategory>
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/rfcomm.h>
-#include <bluetooth/l2cap.h>
 
 #include <errno.h>
 #include <unistd.h>
@@ -67,6 +65,7 @@ QBluetoothSocketPrivate::QBluetoothSocketPrivate()
     : socket(-1),
       socketType(QBluetoothServiceInfo::UnknownProtocol),
       state(QBluetoothSocket::UnconnectedState),
+      socketError(QBluetoothSocket::NoSocketError),
       readNotifier(0),
       connectWriteNotifier(0),
       connecting(false),
@@ -117,7 +116,7 @@ bool QBluetoothSocketPrivate::ensureNativeSocket(QBluetoothServiceInfo::Protocol
     Q_Q(QBluetoothSocket);
     readNotifier = new QSocketNotifier(socket, QSocketNotifier::Read);
     QObject::connect(readNotifier, SIGNAL(activated(int)), q, SLOT(_q_readNotify()));
-    connectWriteNotifier = new QSocketNotifier(socket, QSocketNotifier::Write, q);    
+    connectWriteNotifier = new QSocketNotifier(socket, QSocketNotifier::Write, q);
     QObject::connect(connectWriteNotifier, SIGNAL(activated(int)), q, SLOT(_q_writeNotify()));
 
     connectWriteNotifier->setEnabled(false);
@@ -133,6 +132,12 @@ void QBluetoothSocketPrivate::connectToService(const QBluetoothAddress &address,
     Q_UNUSED(openMode);
     int result = -1;
 
+    if (socket == -1 && !ensureNativeSocket(socketType)) {
+        errorString = QObject::tr("Unknown socket error");
+        q->setSocketError(QBluetoothSocket::UnknownSocketError);
+        return;
+    }
+
     if (socketType == QBluetoothServiceInfo::RfcommProtocol) {
         sockaddr_rc addr;
 
@@ -165,7 +170,7 @@ void QBluetoothSocketPrivate::connectToService(const QBluetoothAddress &address,
         connecting = true;
         q->setSocketState(QBluetoothSocket::ConnectingState);
     } else {
-        errorString = QString::fromLocal8Bit(strerror(errno));
+        errorString = qt_error_string(errno);
         q->setSocketError(QBluetoothSocket::UnknownSocketError);
     }
 }
@@ -178,8 +183,8 @@ void QBluetoothSocketPrivate::_q_writeNotify()
         len = sizeof(errorno);
         ::getsockopt(socket, SOL_SOCKET, SO_ERROR, &errorno, (socklen_t*)&len);
         if(errorno) {
-            errorString = QString::fromLocal8Bit(strerror(errorno));
-            emit q->error(QBluetoothSocket::UnknownSocketError);
+            errorString = qt_error_string(errorno);
+            q->setSocketError(QBluetoothSocket::UnknownSocketError);
             return;
         }
 
@@ -196,13 +201,12 @@ void QBluetoothSocketPrivate::_q_writeNotify()
         }
 
         char buf[1024];
-        Q_Q(QBluetoothSocket);
 
         int size = txBuffer.read(buf, 1024);
 
         if (::write(socket, buf, size) != size) {
-            socketError = QBluetoothSocket::NetworkError;
-            emit q->error(socketError);
+            errorString = QBluetoothSocket::tr("Network Error");
+            q->setSocketError(QBluetoothSocket::NetworkError);
         }
         else {
             emit q->bytesWritten(size);
@@ -215,7 +219,7 @@ void QBluetoothSocketPrivate::_q_writeNotify()
             connectWriteNotifier->setEnabled(false);
             this->close();
         }
-    }    
+    }
 }
 
 // TODO: move to private backend?
@@ -226,23 +230,21 @@ void QBluetoothSocketPrivate::_q_readNotify()
     char *writePointer = buffer.reserve(QPRIVATELINEARBUFFER_BUFFERSIZE);
 //    qint64 readFromDevice = q->readData(writePointer, QPRIVATELINEARBUFFER_BUFFERSIZE);
     int readFromDevice = ::read(socket, writePointer, QPRIVATELINEARBUFFER_BUFFERSIZE);
+    buffer.chop(QPRIVATELINEARBUFFER_BUFFERSIZE - (readFromDevice < 0 ? 0 : readFromDevice));
     if(readFromDevice <= 0){
         int errsv = errno;
         readNotifier->setEnabled(false);
         connectWriteNotifier->setEnabled(false);
-        errorString = QString::fromLocal8Bit(strerror(errsv));
+        errorString = qt_error_string(errsv);
         qCWarning(QT_BT_BLUEZ) << Q_FUNC_INFO << socket << "error:" << readFromDevice << errorString;
         if(errsv == EHOSTDOWN)
-            emit q->error(QBluetoothSocket::HostNotFoundError);
+            q->setSocketError(QBluetoothSocket::HostNotFoundError);
         else
-            emit q->error(QBluetoothSocket::UnknownSocketError);
+            q->setSocketError(QBluetoothSocket::UnknownSocketError);
 
         q->disconnectFromService();
-        q->setSocketState(QBluetoothSocket::UnconnectedState);        
     }
     else {
-        buffer.chop(QPRIVATELINEARBUFFER_BUFFERSIZE - (readFromDevice < 0 ? 0 : readFromDevice));
-
         emit q->readyRead();
     }
 }
@@ -258,16 +260,11 @@ void QBluetoothSocketPrivate::abort()
     // we don't call disconnectFromService or
     // QBluetoothSocket::close
     QT_CLOSE(socket);
-
-    Q_Q(QBluetoothSocket);
-    emit q->disconnected();
+    socket = -1;
 }
 
 QString QBluetoothSocketPrivate::localName() const
 {
-    if (!m_localName.isEmpty())
-        return m_localName;
-
     const QBluetoothAddress address = localAddress();
     if (address.isNull())
         return QString();
@@ -288,9 +285,7 @@ QString QBluetoothSocketPrivate::localName() const
     if (properties.isError())
         return QString();
 
-    m_localName = properties.value().value(QLatin1String("Name")).toString();
-
-    return m_localName;
+    return properties.value().value(QLatin1String("Name")).toString();
 }
 
 QBluetoothAddress QBluetoothSocketPrivate::localAddress() const
@@ -339,9 +334,6 @@ quint16 QBluetoothSocketPrivate::localPort() const
 
 QString QBluetoothSocketPrivate::peerName() const
 {
-    if (!m_peerName.isEmpty())
-        return m_peerName;
-
     quint64 bdaddr;
 
     if (socketType == QBluetoothServiceInfo::RfcommProtocol) {
@@ -361,7 +353,7 @@ QString QBluetoothSocketPrivate::peerName() const
 
         convertAddress(addr.l2_bdaddr.b, bdaddr);
     } else {
-        qCWarning(QT_BT_BLUEZ) << "peerName() called on socket of known type";
+        qCWarning(QT_BT_BLUEZ) << "peerName() called on socket of unknown type";
         return QString();
     }
 
@@ -399,9 +391,7 @@ QString QBluetoothSocketPrivate::peerName() const
     if (properties.isError())
         return QString();
 
-    m_peerName = properties.value().value(QLatin1String("Alias")).toString();
-
-    return m_peerName;
+    return properties.value().value(QLatin1String("Alias")).toString();
 }
 
 QBluetoothAddress QBluetoothSocketPrivate::peerAddress() const
@@ -451,10 +441,18 @@ quint16 QBluetoothSocketPrivate::peerPort() const
 qint64 QBluetoothSocketPrivate::writeData(const char *data, qint64 maxSize)
 {
     Q_Q(QBluetoothSocket);
+
+    if (state != QBluetoothSocket::ConnectedState) {
+        errorString = QBluetoothSocket::tr("Cannot write while not connected");
+        q->setSocketError(QBluetoothSocket::OperationError);
+        return -1;
+    }
+
     if (q->openMode() & QIODevice::Unbuffered) {
         if (::write(socket, data, maxSize) != maxSize) {
-            socketError = QBluetoothSocket::NetworkError;
-            emit q->error(socketError);
+            errorString = QBluetoothSocket::tr("Network Error");
+            q->setSocketError(QBluetoothSocket::NetworkError);
+            return -1;
         }
 
         emit q->bytesWritten(maxSize);
@@ -464,10 +462,10 @@ qint64 QBluetoothSocketPrivate::writeData(const char *data, qint64 maxSize)
     else {
 
         if(!connectWriteNotifier)
-            return 0;
+            return -1;
 
         if(txBuffer.size() == 0) {
-            connectWriteNotifier->setEnabled(true);        
+            connectWriteNotifier->setEnabled(true);
             QMetaObject::invokeMethod(q, "_q_writeNotify", Qt::QueuedConnection);
         }
 
@@ -480,11 +478,19 @@ qint64 QBluetoothSocketPrivate::writeData(const char *data, qint64 maxSize)
 
 qint64 QBluetoothSocketPrivate::readData(char *data, qint64 maxSize)
 {
-    if(!buffer.isEmpty()){
+    Q_Q(QBluetoothSocket);
+
+    if (state != QBluetoothSocket::ConnectedState) {
+        errorString = QBluetoothSocket::tr("Cannot read while not connected");
+        q->setSocketError(QBluetoothSocket::OperationError);
+        return -1;
+    }
+
+    if (!buffer.isEmpty()) {
         int i = buffer.read(data, maxSize);
         return i;
-
     }
+
     return 0;
 }
 
@@ -501,16 +507,7 @@ void QBluetoothSocketPrivate::close()
         connectWriteNotifier->setEnabled(true);
     }
     else {
-
-        delete readNotifier;
-        readNotifier = 0;
-        delete connectWriteNotifier;
-        connectWriteNotifier = 0;
-
-        // We are disconnected now, so go to unconnected.
-        q->setSocketState(QBluetoothSocket::UnconnectedState);
-        emit q->disconnected();
-        ::close(socket);
+        abort();
     }
 
 }
@@ -543,11 +540,6 @@ bool QBluetoothSocketPrivate::setSocketDescriptor(int socketDescriptor, QBluetoo
     return true;
 }
 
-int QBluetoothSocketPrivate::socketDescriptor() const
-{
-    return socket;
-}
-
 qint64 QBluetoothSocketPrivate::bytesAvailable() const
 {
     return buffer.size();
