@@ -76,6 +76,7 @@ public slots:
 private slots:
     void initTestCase();
 
+    void tst_invalidBtAddress();
     void tst_serviceDiscovery_data();
     void tst_serviceDiscovery();
     void tst_serviceDiscoveryAdapters();
@@ -127,9 +128,9 @@ void tst_QBluetoothServiceDiscoveryAgent::initTestCase()
 
         QSignalSpy finishedSpy(&discoveryAgent, SIGNAL(finished()));
         QSignalSpy errorSpy(&discoveryAgent, SIGNAL(error(QBluetoothDeviceDiscoveryAgent::Error)));
-        QSignalSpy discoveredSpy(&discoveryAgent, SIGNAL(deviceDiscovered(const QBluetoothDeviceInfo&)));
-    //    connect(&discoveryAgent, SIGNAL(deviceDiscovered(const QBluetoothDeviceInfo&)),
-    //            this, SLOT(deviceDiscoveryDebug(const QBluetoothDeviceInfo&)));
+        QSignalSpy discoveredSpy(&discoveryAgent, SIGNAL(deviceDiscovered(QBluetoothDeviceInfo)));
+    //    connect(&discoveryAgent, SIGNAL(deviceDiscovered(QBluetoothDeviceInfo)),
+    //            this, SLOT(deviceDiscoveryDebug(QBluetoothDeviceInfo)));
 
         discoveryAgent.start();
 
@@ -149,6 +150,24 @@ void tst_QBluetoothServiceDiscoveryAgent::initTestCase()
     }
 }
 
+void tst_QBluetoothServiceDiscoveryAgent::tst_invalidBtAddress()
+{
+    QBluetoothServiceDiscoveryAgent *discoveryAgent = new QBluetoothServiceDiscoveryAgent(QBluetoothAddress("11:11:11:11:11:11"));
+
+    QCOMPARE(discoveryAgent->error(), QBluetoothServiceDiscoveryAgent::InvalidBluetoothAdapterError);
+    discoveryAgent->start();
+    QCOMPARE(discoveryAgent->isActive(), false);
+    delete discoveryAgent;
+
+    discoveryAgent = new QBluetoothServiceDiscoveryAgent(QBluetoothAddress());
+    QCOMPARE(discoveryAgent->error(), QBluetoothServiceDiscoveryAgent::NoError);
+    if (QBluetoothLocalDevice::allDevices().count() > 0) {
+        discoveryAgent->start();
+        QCOMPARE(discoveryAgent->isActive(), true);
+    }
+    delete discoveryAgent;
+}
+
 void tst_QBluetoothServiceDiscoveryAgent::serviceDiscoveryDebug(const QBluetoothServiceInfo &info)
 {
     qDebug() << "Discovered service on"
